/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner" 

#ifndef THREAD_SAFE_OBJECT_POOL_H
#define THREAD_SAFE_OBJECT_POOL_H

#include <support/Locker.h>
#include "pool/ObjectPool.h"

//:Same as your average ObjectPool, but uses a BLocker to serialize operations,
//:so that it may be safely shared between threads.  Only compiles under BeOS!
template <class Object> class ThreadSafeObjectPool : public ObjectPool<Object>
{
public:
   ThreadSafeObjectPool(uint32 maxPoolSize=100,
              ObjectCallback recycleCallback = NULL, void * recycleData = NULL,
              ObjectCallback initCallback = NULL, void * initData = NULL) :
      ObjectPool<Object>(maxPoolSize, recycleCallback, recycleData, initCallback, initData) {/* empty */}
   virtual ~ThreadSafeObjectPool() {/* empty */}

protected:
   virtual void Lock() {_locker.Lock();}
   virtual void Unlock() {_locker.Unlock();}

private:
   BLocker _locker;
};

#endif
