/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef SOCKET_DATA_IO
#define SOCKET_DATA_IO

#include <stdio.h>
#include "dataio/PortableDataIO.h"

//: Data I/O to and from a stdio FILE 
class FileDataIO : public PortableDataIO
{
public:
   // (file) becomes property of this FileDataIO object and will be closed 
   // when this object is deleted.
   FileDataIO(FILE * file) : _file(file) {NoteAllocated(this, "FileDataIO");}
   virtual ~FileDataIO() {fclose(_file); NoteDeleted(this, "FileDataIO");}

   // Reads bytes from our file and places them into (buffer).
   // See PortableDataIO::Read() for semantics.
   virtual ssize_t Read(void * buffer, size_t size)  
   {
      ssize_t ret = fread(buffer, 1, size, _file);
      return (ret > 0) ? ret : -1;  // EOF is an error, and it's returned as zero
   }

   // Takes bytes from (buffer) and writes them out to our file.
   // See PortableDataIO::Write() for semantics.
   virtual ssize_t Write(const void * buffer, size_t size)
   {
      ssize_t ret = fwrite(buffer, 1, size, _file);
      return (ret > 0) ? ret : -1;   // zero is an error
   }

   // Flushes the output with fflush()
   virtual void FlushOutput() {fflush(_file);}
   
private:
   FILE * _file;
};

#endif
