/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <stdlib.h>
#include "reflector/ReflectServer.h"
#include "reflector/DumbReflectSession.h"
#include "reflector/StorageReflectSession.h"
#include "stringtokenizer/StringTokenizer.h"

#define DEFAULT_MUSCLED_PORT 2960

int main(int argc, char ** argv) 
{
   int port = (argc > 1) ? atoi(argv[1]) : 0;
   uint32 maxBytes = (uint32)-1L;

   // Parse arguments
   for (int i=1; i<argc; i++)
   {
      StringTokenizer tok(argv[i], "=");
      const char * keyword = tok.GetNextToken();

      if (keyword)
      {
         const char * value = tok.GetNextToken();

         while(keyword[0] == '-') keyword++;  // stupid Unix geeks anyway
         if (strcmp(keyword, "help") == 0)
         {
            LogTime(LOG_INFO, "Usage:  ./muscled [port=2960] [maxmem=512]\n");
            exit(5);
         }
         else if (value)
         {
                 if (strcmp(keyword, "port") == 0) port = atoi(value);
            else if (strcmp(keyword, "maxmem") == 0) 
            {
               int megs = atoi(value);
               if (megs < 1) megs = 1;
               LogTime(LOG_INFO, "Limiting memory usage to %i megabyte%s.\n", megs, (megs==1)?"":"s");
               maxBytes = megs*1024L*1024L;
            }
            else LogTime(LOG_WARNING, "Unknown argument [%s] not parsed.\n", keyword);
         }
         else 
         {
            // legacy parsing support
            if ((keyword[0] >= '0')&&(keyword[0] <= '9')) port = atoi(keyword);
            else LogTime(LOG_WARNING, "Argument [%s] not parsed.  Run ./muscled help for more info.\n", keyword);
         }
      }
   }

   AddOutOfMemoryHandler((OutOfMemoryHandlerFunc)(StorageReflectSession::DrainPools));

   int retVal = 0;
   ReflectServer server(maxBytes);
   if (server.ServerProcessLoop(StorageSessionCreationFunc, (port > 0) ? port : DEFAULT_MUSCLED_PORT) == B_NO_ERROR) LogTime(LOG_INFO, "Server process exiting\n");
   else 
   {
      LogTime(LOG_CRITICALERROR, "Server process aborted\n");
      retVal = 10;
   }
   server.Cleanup();
   return retVal;
}
