/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"   

/*******************************************************************************
/
/   File:      PortablePoint.h
/
/   Description:    Portable version of Be's PortablePoint class
/
*******************************************************************************/

#ifndef PORTABLE_POINT_H
#define PORTABLE_POINT_H

#include <stdio.h>
#include "support/PortableFlattenable.h"
#include "support/PortableTypeConstants.h"

/*----------------------------------------------------------------------*/
/*----- PortablePoint class --------------------------------------------*/

//:A portable version of Be's BPoint class.
class PortablePoint : public PortableFlattenable
{
public:
   float x;
   float y;

   PortablePoint() {NoteAllocated(this, "PortablePoint"); x = y = 0.0f;}
   PortablePoint(float X, float Y) {NoteAllocated(this, "PortablePoint"); Set(X, Y);}
   PortablePoint(const PortablePoint & rhs) : PortableFlattenable() {NoteAllocated(this, "PortablePoint"); *this = rhs;}
   virtual ~PortablePoint() {NoteDeleted(this, "PortablePoint");}
 
   void Set(float X, float Y) {x = X; y = Y;}

   void ConstrainTo(PortablePoint topLeft, PortablePoint bottomRight)
   {
      if (x < topLeft.x) x = topLeft.x;
      if (y < topLeft.y) y = topLeft.y;
      if (x > bottomRight.x) x = bottomRight.x;
      if (y > bottomRight.y) y = bottomRight.y;
   }

   void PrintToStream() const
   {
      printf("PortablePoint: %f %f\n", x, y);
   }
      
   PortablePoint operator+(const PortablePoint&p) const
   {
      return PortablePoint(x+p.x, y+p.y);
   }

   PortablePoint operator-(const PortablePoint&p) const
   {
      return PortablePoint(x-p.x, y-p.y);
   }

   PortablePoint& operator+=(const PortablePoint&p)
   {
      x += p.x;
      y += p.y;
      return *this;
   }

   PortablePoint& operator-=(const PortablePoint&p)
   {
      x -= p.x;
      y -= p.y;
      return *this;
   }

   bool operator!=(const PortablePoint&p) const
   {
      return ((x != p.x)||(y != p.y));
   }

   bool operator==(const PortablePoint&p) const
   {
      return ((x == p.x)&&(y == p.y));
   }

   virtual bool IsFixedSize() const {return true;} 
   virtual type_code TypeCode() const {return B_POINT_TYPE;}
   virtual size_t FlattenedSize() const {return 2 * sizeof(float);}

   virtual void Flatten(uint8 * buffer) const 
   {
      float * buf = (float *) buffer;
      buf[0] = B_HOST_TO_LENDIAN_FLOAT(x);
      buf[1] = B_HOST_TO_LENDIAN_FLOAT(y);
   }

   virtual status_t Unflatten(const uint8 * buffer, size_t size) 
   {
      if (size >= FlattenedSize())
      {
         float * buf = (float *) buffer;
         x = B_LENDIAN_TO_HOST_FLOAT(buf[0]);
         y = B_LENDIAN_TO_HOST_FLOAT(buf[1]);
         return B_NO_ERROR;
      }
      else return B_ERROR;
   }
};

#endif 
