/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>

#include "message/PortableMessage.h"
#include "besupport/ConvertMessages.h"

#define TEST(x) if ((x) != B_NO_ERROR) printf("Operation failed, line %li\n", __LINE__);
#define NEGATIVETEST(x) if ((x) == B_NO_ERROR) printf("Operation succeeded when it shouldn't have, line %li\n", __LINE__);

// This program tests the PortableMessage <-> BMessage conversion functions in the besupport directory.
void main(void) 
{
   PortableMessage msg('HELO');
   TEST(msg.AddString("Hi", "there"));
   TEST(msg.AddString("Friesner", "Jeremy"));
   TEST(msg.AddString("Friesner", "Joanna"));
   TEST(msg.AddString("Friesner", "Joellen"));
   TEST(msg.AddString("Chicken", "Soup"));
   TEST(msg.AddString("Chicken", "Vegetable"));
   TEST(msg.AddString("Chicken", "Lips"));
   TEST(msg.AddString("Fred", "Flintstone"));
   TEST(msg.AddPoint("pointMe", PortablePoint(1,2)));
   TEST(msg.AddRect("rectMe", PortableRect(1,2,3,4)));
   TEST(msg.AddRect("rectMe", PortableRect(2,3,4,5)));
   TEST(msg.AddData("Data", B_RAW_TYPE, "Keyboard", 9));
   TEST(msg.AddData("Data", B_RAW_TYPE, "BLACKJACK", 2));

   PortableMessage subMsg('SUBm');
   PortableMessage deeper('Deep');
   subMsg.AddMessage("Russian Dolls", deeper);
   TEST(msg.AddMessage("TestMessage", subMsg));

   for (int i=0; i<10; i++) TEST(msg.AddInt8("TestInt8", i));
   for (int i=0; i<12; i++) TEST(msg.AddInt16("TestInt16", i));
   for (int i=0; i<13; i++) TEST(msg.AddInt32("TestInt32", i));
   for (int i=0; i<11; i++) TEST(msg.AddInt64("TestInt64", i));
   for (int i=0; i<5; i++) TEST(msg.AddDouble("TestDouble", i));
   for (int i=0; i<6; i++) TEST(msg.AddFloat("TestFloat", i));
   for (int i=0; i<25; i++) TEST(msg.AddBool("TestBool", i));

   printf("ORIGINAL MESSAGE:\n");
   msg.PrintToStream();

   BMessage bMsg;
   printf("CONVERTING TO BMESSAGE...\n");
   TEST(ConvertPortableMessageToBMessage(msg, bMsg));
   bMsg.PrintToStream();

   printf("CONVERTING BACK TO PORTABLEMESSAGE...\n");
   PortableMessage portableMsg;
   TEST(ConvertBMessageToPortableMessage(bMsg, portableMsg));
   portableMsg.PrintToStream();

   PortableMessage rSub, rDeep;
   if ((portableMsg.FindMessage("TestMessage", rSub) == B_NO_ERROR)&&
       (rSub.FindMessage("Russian Dolls", rDeep) == B_NO_ERROR)) 
   { 
      printf("Nested messages are:\n");
      rSub.PrintToStream();
      rDeep.PrintToStream();
   }
   else printf("ERROR RE-READING NESTED MESSAGES!\n");

   printf("TORTURE TEST!\n");
   int i=1000;
   size_t origSize = portableMsg.FlattenedSize();
   while(i--)
   {
      TEST(ConvertBMessageToPortableMessage(bMsg, portableMsg));
      TEST(ConvertPortableMessageToBMessage(portableMsg, bMsg));
      size_t flatSize = portableMsg.FlattenedSize();
      if (flatSize != origSize) printf("ERROR, FLATTENED SIZE CHANGED %lu -> %lu\n", origSize, flatSize);
      uint8 * buf = new uint8[flatSize]; 
      portableMsg.Flatten(buf);
      TEST(portableMsg.Unflatten(buf, flatSize));
      delete [] buf;
   } 
}
