/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

/******************************************* 
   Downloaded from: http://www.mike95.com 
   Copyright (c)1997 Michael Olivero 
   All Rights Reserved 
********************************************/ 
#include "PortableString.h"
#include <ctype.h>
#include <stdio.h> 
#include <stdlib.h> 
#include <stdarg.h>

//============
//Constructors
//============
PortableString::PortableString(const char * str) : _buffer(NULL), _bufferLen(0), _length(0)
{
   NoteAllocated(this, "PortableString");
   *this = str;
}

PortableString::PortableString(const PortableString & str) : PortableFlattenable(), _buffer(NULL), _bufferLen(0), _length(0)
{
   NoteAllocated(this, "PortableString");
   *this = str;
}

char
PortableString::CharAt(uint32 loc) const
{
   return operator[](loc);
}

int
PortableString::CompareTo(const PortableString &s2) const
{
   return strcmp(Cstr(), s2.Cstr());
}

const PortableString &
PortableString::operator=(const PortableString & rhs)
{
   if (this != &rhs)
   {
      uint32 rhsLen = rhs.Length();
      if (rhsLen > 0)
      {
         if (EnsureBufferSize(rhsLen+1, false) == B_NO_ERROR)
         {
            strcpy(_buffer, rhs.Cstr());
            _length = rhsLen;
         }
      }
      else
      {
         if (_buffer) _buffer[0] = '\0';
         _length = 0;
      }
   }
   return *this;
}

const PortableString &
PortableString::operator=(const char * val)
{
   if ((val)&&(val[0]))
   {
      int vlen = strlen(val);
      if (EnsureBufferSize(vlen+1, false) == B_NO_ERROR)
      {
         strcpy(_buffer, val);
         _length = vlen;
      }
   }
   else
   {
      if (_buffer) _buffer[0] = '\0';
      _length = 0;
   }
   return *this;
}

const PortableString &
PortableString::operator+=(const char aChar)
{
   if (EnsureBufferSize(Length()+2, true) == B_NO_ERROR)
   {
      _buffer[_length++] = aChar;
      _buffer[_length]   = '\0';
   }
   return *this;
}

PortableString
PortableString :: Prepend(const PortableString & str) const
{
   return str + (*this);
}

PortableString 
PortableString :: Append(const PortableString & str) const
{
   return (*this) + str;
}

const PortableString &
PortableString::operator+=(const PortableString &other)
{
   uint32 otherLen = other.Length();
   if ((otherLen > 0)&&(EnsureBufferSize(Length() + otherLen + 1, true) == B_NO_ERROR))
   {
      strcpy(&_buffer[_length], other.Cstr());
      _length += otherLen;
   }
   return *this;
}
 

int
PortableString::operator==(const PortableString &rhs) const
{
   if (Length() != rhs.Length()) return 0;
   return (strcmp(Cstr(), rhs.Cstr()) == 0);
}

int
PortableString::operator!=(const PortableString &rhs) const
{
   return !(*this == rhs);
}

int
PortableString::operator<(const PortableString &rhs) const
{
   return strcmp(Cstr(), rhs.Cstr()) < 0;
}

int
PortableString::operator>(const PortableString &rhs) const
{
   return strcmp(Cstr(), rhs.Cstr()) > 0;
}

int
PortableString::operator<=(const PortableString &rhs) const
{
   return strcmp(Cstr(), rhs.Cstr()) <= 0;
}

int
PortableString::operator>=(const PortableString & rhs) const
{
   return strcmp(Cstr(), rhs.Cstr()) >= 0;
}

char &
PortableString::operator[](uint32 Index)
{
   verifyIndex(Index);
   return _buffer[Index];
}

char
PortableString::operator[](uint32 Index) const
{
   verifyIndex(Index);
   return _buffer[Index];
}


int
PortableString::EndsWith(const PortableString &s2) const
{ 
   return (Length() < s2.Length()) ? 0 : (strcmp(Cstr()+(Length()-s2.Length()), s2.Cstr()) == 0); 
}

int
PortableString::Equals(const PortableString &s2) const
{
   return (*this == s2);
}


int
PortableString::EqualsIgnoreCase(const PortableString &s2) const
{ 
   if (Length() != s2.Length()) return 0;
   if (this == &s2) return 1; 
   return (strcmp(ToLowerCase().Cstr(), s2.ToLowerCase().Cstr()) == 0);
}
 
void
PortableString::Replace(char findChar, char replaceChar)
{
   if ((_buffer)&&(findChar != replaceChar))
   {
      char * c = _buffer;
      while(*c)
      {
         if (*c == findChar) *c = replaceChar;
         c++;
      }
   }
}
   
void
PortableString::Replace(const PortableString& match, const PortableString& replace)
{
   if (match != replace)
   {
      PortableString temp(*this);
      PortableString newString;
 
      int loc; 
      while ((loc = temp.IndexOf(match)) != -1)
      {
         newString += temp.Substring(0, loc);
         newString += replace;
         temp = temp.Substring(loc + match.Length());
      } 
      newString += temp;
      *this = newString;
   }
}

int
PortableString::IndexOf(char temp) const
{ 
   return IndexOf(temp, 0);
}

int
PortableString::IndexOf(char ch, uint32 fromIndex) const
{
   const char * temp = (fromIndex < Length()) ? strchr(Cstr()+fromIndex, ch) : NULL; 
   return temp ? (temp - Cstr()) : -1; 
}

int
PortableString::IndexOf(const PortableString &s2) const
{
   return IndexOf(s2, 0);
}


int
PortableString::IndexOf(const PortableString &s2, uint32 fromIndex) const
{
   const char *theFind = (fromIndex < Length()) ? strstr(Cstr()+fromIndex, s2.Cstr()) : NULL;
   return theFind ? (theFind - Cstr()) : -1;
}


int
PortableString::LastIndexOf(char theChar) const
{
   return LastIndexOf(theChar, Length() - 1);
}

int
PortableString::LastIndexOf(char ch, uint32 fromIndex) const
{
   const char * lio = (fromIndex < Length()) ? strrchr(Cstr()+fromIndex, ch) : NULL;
   return lio ? (lio - Cstr()) : -1;
}

int
PortableString::LastIndexOf(const PortableString &s2) const
{
   return (s2.Length() <= Length()) ? LastIndexOf(s2, Length() - s2.Length()) : -1;
}

int
PortableString::LastIndexOf(const PortableString &s2, uint32 fromIndex) const
{
   if (s2.Length() == 0) return Length()-1;
   if (fromIndex >= Length()) return -1;
   for (int i=fromIndex; i>=0; i--) if (strncmp(Cstr()+i, s2.Cstr(), s2.Length()) == 0) return i;
   return -1;
}

int
PortableString::StartsWith(const PortableString &s2) const
{
   return (Length() < s2.Length()) ? 0 : StartsWith(s2, 0); 
} 
 
int 
PortableString::StartsWith(const PortableString &s2, uint32 offset) const 
{
   return (offset > Length() - s2.Length()) ? 0 : (strncmp(Cstr()+offset, s2.Cstr(), s2.Length()) == 0); 
}

PortableString
PortableString::Substring(uint32 left) const
{
   return Substring(left, Length());
}

PortableString
PortableString::Substring(uint32 left, uint32 right) const
{
   MASSERT(left <= right, "Invalid Substring range");
   MASSERT(right <= Length(), "Index out of bounds");

   PortableString ret;
   if (ret.EnsureBufferSize(right-left+1, true) == B_NO_ERROR)
   {
      const char * c = Cstr();
      for (uint32 i=left; i<right; i++) ret += c[i];
   }
   return ret;
}

PortableString
PortableString::ToLowerCase() const
{
   PortableString ret(_buffer);
   if (ret._buffer) for (uint32 i=0; i<ret.Length(); i++) ret._buffer[i] = tolower(ret._buffer[i]);
   return ret;
}

PortableString
PortableString::ToUpperCase() const
{
   PortableString ret(_buffer);
   if (ret._buffer) for (uint32 i=0; i<ret.Length(); i++) ret._buffer[i] = toupper(ret._buffer[i]);
   return ret;
}
 
PortableString 
PortableString::Trim() const 
{ 
   PortableString temp(Cstr());
   uint32 i,j; 
 
   for (i = 0; i < Length(); i++) if (!isspace(Cstr()[i])) break; 
   for (j = temp.Length() - 1; j > i; j--) if (!isspace(Cstr()[j])) break; 
   return temp.Substring(i, j + 1); 
} 

size_t PortableString :: FlattenedSize() const
{
   return Length()+1;   
}

void PortableString :: Flatten(uint8 *buffer) const
{
   strcpy((char *)buffer, Cstr());
}

status_t PortableString :: Unflatten(const uint8 *buf, size_t /*size*/)
{
   const char * str = (const char *) buf;
   *this = str;
   return B_NO_ERROR;  
}

uint32 PortableString :: HashCode() const
{
   return CStringHashFunc(Cstr());
}

uint32 PortableStringHashFunc(const PortableString & pstr)
{ 
   return pstr.HashCode();
}

/*--- ElfHash --------------------------------------------------- 
 *  The published hash algorithm used in the UNIX ELF format 
 *  for object files. Accepts a pointer to a string to be hashed 
 *  and returns an unsigned long. 
 *  jaf:  stolen from: http://www.ddj.com/articles/1996/9604/9604b/9604b.htm?topic=algorithms
 *-------------------------------------------------------------*/ 
uint32 CStringHashFunc(const char * const & n)  // if you can read that, you know too much C++
{
    uint32 h = 0, g; 
    const unsigned char * name = (const unsigned char *) n;
    while (*name) 
    { 
        h = (h << 4) + *name++; 
        if ((g = h & 0xF0000000) != 0) h ^= g >> 24; 
        h &= ~g; 
    } 
    return h; 
}

bool CStringCompareFunc(const char * const & s1, const char * const & s2)
{
    return(strcmp(s1, s2) == 0);
}
