/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

/* 
   This library was downloaded from: http://www.mike95.com 
 
   This library is copyright.  It may freely be used for personal purposes  
   if the restriction listed below is adhered to. 
       Author: Michael Olivero 
       Email:  mike95@mike95.com 
 
   //=============================== 
   //Start of Restriction Definition 
   //=============================== 
   Anyone can have full use of the library provided they keep this complete comment 
   with the source.  Also I would like to ask if any changes are made to the 
   code for efficiency reasons, please let me know so I may look into your change and 
   likewise incorporate it into library.  If a suggestion makes it into the library, 
   your credits will be added to this information. 
 
   Authors of Computer related books are welcome to include this source code as part 
   of their publishing, provided credit the Author and make note of where the source 
   code was obtained from: http://www.mike95.com 
   //============================= 
   //End of Restriction Definition 
   //============================= 
 
 
   Description: 
   Visit http://www.mike95.com/c_plusplus/classes/JString/ 

   This library avoids the need to use pointers to char or manual
   memory manipulation while working with Strings.  This class has been
   based on the Java String class, and thus has all of the Java public functionality 
   with a few extras [replace()] needed for useful functionality.
 
   //The following people have contributed to the solution 
   //of bugs in this library 
   //===================================================== 
   //Carl Pupa, email: pumacat@erols.com 
*/
 
#ifndef __PortableString
#define __PortableString

#define SMALL_PORTABLE_STRING_LENGTH 7  // strings shorter than this length can be stored inline, without requiring an extra new[].

#include <new.h>
#include <string.h> 
#include <stdio.h> 
#include "support/PortableMacros.h"
#include "support/PortableTypeDefs.h"
#include "support/PortableTypeConstants.h"
#include "support/PortableFlattenable.h"

class PortableString;

//:A nice hashing function for PortableStrings.
uint32 PortableStringHashFunc(const PortableString &);

//:A function for hashing (const char *)'s
uint32 CStringHashFunc(const char * const &);

//:A function for comparing (const char *)'s with strcmp().
bool CStringCompareFunc(const char * const &, const char * const &);

//:Character String class, similar to Java's java.lang.String
class PortableString : public PortableFlattenable {
public:
   // Constructor(s) / Destructors
   //-----------------------------
   PortableString(const char * str = NULL);
   PortableString(const PortableString & str);
   virtual ~PortableString() {if (_buffer != _smallBuffer) delete [] _buffer; NoteDeleted(this, "PortableString");}

   //Operators
   //----------
   const PortableString &   operator = (const char * val);
   const PortableString &   operator = (const PortableString &Rhs);
   const PortableString &   operator +=(const PortableString &Rhs);
   const PortableString &   operator +=(const char);
   int            operator ==(const PortableString &Rhs) const;
   int            operator !=(const PortableString &Rhs) const;
   int            operator < (const PortableString &Rhs) const;
   int            operator > (const PortableString &Rhs) const;
   int            operator <=(const PortableString &Rhs) const;
   int            operator >=(const PortableString &Rhs) const;
   char           operator [](uint32 Index) const;
   char&          operator [](uint32 Index);

   //Method
   //-------
   //INSPECTORS
   //==========
   char           CharAt(uint32 index) const;
   int            CompareTo(const PortableString &anotherString) const;
   const char*    Cstr() const {return _buffer ? _buffer : "";}
   int            EndsWith(const PortableString &suffix) const;
   int            Equals(const PortableString &anObject) const;
   int            EqualsIgnoreCase(const PortableString &anotherString) const;
   int            IndexOf(char ch) const;
   int            IndexOf(char ch, uint32 fromIndex) const;
   int            IndexOf(const PortableString &str) const;
   int            IndexOf(const PortableString &str, uint32 fromIndex) const;
   int            LastIndexOf(char ch) const;
   int            LastIndexOf(char ch, uint32 fromIndex) const;
   int            LastIndexOf(const PortableString &str) const;
   int            LastIndexOf(const PortableString &str, uint32 fromIndex) const;
   const uint32   Length() const { return _length; }
   int            StartsWith(const PortableString &prefix) const;
   int            StartsWith(const PortableString &prefix, uint32 toffset) const; 
   PortableString         Prepend(const PortableString & str) const;
   PortableString         Append(const PortableString & str) const;
   PortableString         Substring(uint32 beginIndex) const; 
   PortableString         Substring(uint32 beginIndex, uint32 endIndex) const; 
   PortableString         ToLowerCase() const; 
   PortableString         ToUpperCase() const; 
   PortableString         Trim() const;  

   // Returns a hash code for this string (hash values are demand-cached for efficiency)
   uint32 HashCode() const;

   //Methods 
   //------- 
   //MODIFIERS 
   //========= 
   void         Replace(char oldChar, char newChar); 
   void         Replace(const PortableString& match, const PortableString& replace); 
 
   // Friends 
   //-------- 
   //friend PortableString operator + (const PortableString &Lhs, const PortableString &Rhs); 

   // PortableFlattenable interface
   virtual bool            IsFixedSize() const {return false;}
   virtual type_code       TypeCode() const {return B_STRING_TYPE;}
   virtual size_t          FlattenedSize() const;
   virtual void            Flatten(uint8 *buffer) const;
   virtual status_t        Unflatten(const uint8 *buf, size_t size);

protected:
   //Members
   //------- 
   char _smallBuffer[SMALL_PORTABLE_STRING_LENGTH+1];  // store very small strings in-line, to avoid dynamic allocation
   char * _buffer;            // Stores the chars.  May point at (_smallBuffer), or a dynamically allocated buffer
   uint32 _bufferLen;         // Number of bytes pointed to by (_buffer)
   uint32 _length;            // cached strlen(_buffer)

   status_t EnsureBufferSize(uint32 newBufLen, bool retainValue);
   void verifyIndex(uint32 number) const;
};

// This method tries to ensure that at least (newBufLen) chars
// are available for storing data in.  (requestedBufLen) should include
// the terminating NUL.  If (retainValue) is true, the current string value
// will be retained; otherwise it should be set right after this call
// returns...
inline status_t
PortableString::EnsureBufferSize(uint32 requestedBufLen, bool retainValue)
{
   if ((requestedBufLen > 0)&&((_buffer == NULL)||(requestedBufLen > _bufferLen)))
   {
      // If we're doing an initial allocation, just allocate the bytes requested.
      // If it's a re-allocation, allocate more than requested as it's more likely
      // to happen yet another time...
      uint32 newBufLen = (_buffer == NULL) ? requestedBufLen : (requestedBufLen * 2);
      char * newBuf = (requestedBufLen <= sizeof(_smallBuffer)) ? _smallBuffer : newnothrow char[newBufLen]; 
      if (newBuf == _smallBuffer) newBufLen = sizeof(_smallBuffer);
      if (newBuf)
      {
         if ((retainValue)&&(_buffer)&&(newBuf != _buffer)) strncpy(newBuf, _buffer, newBufLen);
         if (_buffer != _smallBuffer) delete [] _buffer;
         _buffer = newBuf;
         _bufferLen = newBufLen;
      }
      else 
      {
         WARN_OUT_OF_MEMORY;
         return B_ERROR;
      }
   }
   return B_NO_ERROR;
}

inline void 
PortableString::verifyIndex(uint32 index) const 
{ 
   MASSERT(index < _length, "Index Out Of Bounds Exception"); 
} 

//Friend Functions
//================
inline const PortableString
operator+(const PortableString & Lhs, const PortableString &Rhs) 
{
   PortableString ret(Lhs);
   ret += Rhs;
   return ret; 
}

#endif
